@@ -1,5 +1,6 @@
-from .linq_util import (Where, Select, SelectMany, First, Last, ElementAt, Aggregate, Count, Max,
-                        Min, Sum, Zip, OrderBy, Choose)
+from .linq_util import (Where, Select, SelectMany, First, Last, ElementAt, Contains, Aggregate,
+                        Count, Max, Min, Sum, All, Any, Concat, Zip, OrderBy, OrderByDescending,
+                        Choose)
 from .ast_util import wrap_ast, unwrap_ast
 
 import lark
@@ -191,6 +192,11 @@ class PythonASTToTextASTTransformer(ast.NodeVisitor):
                                                self.visit(node.source),
                                                self.visit(node.index))
 
+    def visit_Contains(self, node):
+        return self.make_composite_node_string('Contains',
+                                               self.visit(node.source),
+                                               self.visit(node.value))
+
     def visit_Aggregate(self, node):
         return self.make_composite_node_string('Aggregate',
                                                self.visit(node.source),
@@ -209,6 +215,21 @@ class PythonASTToTextASTTransformer(ast.NodeVisitor):
     def visit_Sum(self, node):
         return self.make_composite_node_string('Sum', self.visit(node.source))
 
+    def visit_All(self, node):
+        return self.make_composite_node_string('All',
+                                               self.visit(node.source),
+                                               self.visit(node.predicate))
+
+    def visit_Any(self, node):
+        return self.make_composite_node_string('Any',
+                                               self.visit(node.source),
+                                               self.visit(node.predicate))
+
+    def visit_Concat(self, node):
+        return self.make_composite_node_string('Concat',
+                                               self.visit(node.first),
+                                               self.visit(node.second))
+
     def visit_Zip(self, node):
         return self.make_composite_node_string('Zip', self.visit(node.source))
 
@@ -217,6 +238,11 @@ class PythonASTToTextASTTransformer(ast.NodeVisitor):
                                                self.visit(node.source),
                                                self.visit(node.key_selector))
 
+    def visit_OrderByDescending(self, node):
+        return self.make_composite_node_string('OrderByDescending',
+                                               self.visit(node.source),
+                                               self.visit(node.key_selector))
+
     def visit_Choose(self, node):
         return self.make_composite_node_string('Choose',
                                                self.visit(node.source),
@@ -431,6 +457,11 @@ class TextASTToPythonASTTransformer(lark.Transformer):
                 raise SyntaxError('ElementAt node must have two fields; found ' + str(len(fields)))
             return ElementAt(source=fields[0], index=fields[1])
 
+        elif node_type == 'Contains':
+            if len(fields) != 2:
+                raise SyntaxError('Contains node must have two fields; found ' + str(len(fields)))
+            return Contains(source=fields[0], value=fields[1])
+
         elif node_type == 'Aggregate':
             if len(fields) != 3:
                 raise SyntaxError('Aggregate node must have three fields; found '
@@ -462,6 +493,33 @@ class TextASTToPythonASTTransformer(lark.Transformer):
                 raise SyntaxError('Sum node must have one field; found ' + str(len(fields)))
             return Sum(source=fields[0])
 
+        elif node_type == 'All':
+            if len(fields) != 2:
+                raise SyntaxError('All node must have two fields; found ' + str(len(fields)))
+            if not isinstance(fields[1], ast.Lambda):
+                raise SyntaxError('All predicate must be a lambda; found '
+                                  + str(type(fields[1])))
+            if len(fields[1].args.args) != 1:
+                raise SyntaxError('All predicate must have exactly one argument; found '
+                                  + str(len(fields[1].args.args)))
+            return All(source=fields[0], predicate=fields[1])
+
+        elif node_type == 'Any':
+            if len(fields) != 2:
+                raise SyntaxError('Any node must have two fields; found ' + str(len(fields)))
+            if not isinstance(fields[1], ast.Lambda):
+                raise SyntaxError('Any predicate must be a lambda; found '
+                                  + str(type(fields[1])))
+            if len(fields[1].args.args) != 1:
+                raise SyntaxError('Any predicate must have exactly one argument; found '
+                                  + str(len(fields[1].args.args)))
+            return Any(source=fields[0], predicate=fields[1])
+
+        elif node_type == 'Concat':
+            if len(fields) != 2:
+                raise SyntaxError('Concat node must have two fields; found ' + str(len(fields)))
+            return Concat(first=fields[0], second=fields[1])
+
         elif node_type == 'Zip':
             if len(fields) != 1:
                 raise SyntaxError('Zip node must have one field; found ' + str(len(fields)))
@@ -478,6 +536,18 @@ class TextASTToPythonASTTransformer(lark.Transformer):
                                   + str(len(fields[1].args.args)))
             return OrderBy(source=fields[0], key_selector=fields[1])
 
+        elif node_type == 'OrderByDescending':
+            if len(fields) != 2:
+                raise SyntaxError('OrderByDescending node must have two fields; found '
+                                  + str(len(fields)))
+            if not isinstance(fields[1], ast.Lambda):
+                raise SyntaxError('OrderByDescending key selector must be a lambda; found '
+                                  + str(type(fields[1])))
+            if len(fields[1].args.args) != 1:
+                raise SyntaxError('OrderByDescending key selector must have exactly one argument;'
+                                  + ' found ' + str(len(fields[1].args.args)))
+            return OrderByDescending(source=fields[0], key_selector=fields[1])
+
         elif node_type == 'Choose':
             if len(fields) != 2:
                 raise SyntaxError('Choose node must have two fields; found ' + str(len(fields)))
